// Operating with mozgVector and mozgMatrix classes objects:
// making, deleting, adding, out-product, inner-product,
// sorting (increasing)...

#include "mozg/VectorMatrix.hh"


#ifdef Mozg_BEFORE_EXCEPTION
  char mozg_VecMatr_func_name[500];
#endif

// multiply each element of first vector by relevant element of second one
template <class vec_t, class vec_size_t> mozgVector<vec_t,vec_size_t>& 
mozgVector<vec_t,vec_size_t>::operator *= (const mozgVector& vector2)
  {
    vec_t* temp1 = vector,
         * temp2 = vector2.vector;

    vec_size_t tmp = size ();

#ifdef Mozg_BEFORE_EXCEPTION
  if (size () != vector2.size ())
    before_exception ("mozgVector& operator *= (const mozgVector& vector2)\n"
		      "     this and vector2 SIZES MISMATCH !!!");
#endif

    while (tmp--) *temp1++ *= *temp2++;

    return *this;
  }
template mozgVector<mozgflt,mozgint>&
mozgVector<mozgflt,mozgint>::operator *= (const mozgVector<mozgflt,mozgint>&);

template mozgVector<mozgint,mozgint>&
mozgVector<mozgint,mozgint>::operator *= (const mozgVector<mozgint,mozgint>&);

template <class vec_t, class vec_size_t> mozgVector<vec_t,vec_size_t>& 
mozgVector<vec_t,vec_size_t>::operator /= (const vec_t number)
  {
    vec_t* temp = vector;

    vec_size_t tmp = size ();

    while (tmp--) *temp++ /= number;

    return *this;
  }
template mozgVector<mozgflt,mozgint>&
mozgVector<mozgflt,mozgint>::operator /= (const mozgflt);

template mozgVector<mozgint,mozgint>&
mozgVector<mozgint,mozgint>::operator /= (const mozgint);


template <class vec_t, class vec_size_t> mozgVector<vec_t,vec_size_t>& 
mozgVector<vec_t,vec_size_t>::operator /= (const mozgVector& denominator)
  {
    vec_t* temp1 = vector,
         * temp2 = denominator.vector;

    vec_size_t tmp = size ();

#ifdef Mozg_BEFORE_EXCEPTION
  if (size () != denominator.size ())
    before_exception ("mozgVector& operator /= "
                      "(const mozgVector& denominator)\n"
		      "     this and denominator SIZES MISMATCH !!!");
#endif

    while (tmp--) *temp1++ /= *temp2++;

    return *this;
  }
template mozgVector<mozgflt,mozgint>&
mozgVector<mozgflt,mozgint>::operator /= (const mozgVector<mozgflt,mozgint>&);

template mozgVector<mozgint,mozgint>&
mozgVector<mozgint,mozgint>::operator /= (const mozgVector<mozgint,mozgint>&);

// It is overloading "file_stream << [mozgVector object]"
template <class vec_t, class vec_size_t>
ofstream& operator << (ofstream& output,
		       const mozgVector<vec_t,vec_size_t>& vector)
{
  vec_size_t size = vector.size ();

  vec_t* temp = vector.vec ();

  while (size--)
    output << ' ' << (*temp++);

  return output;
}
template ofstream& operator << (ofstream& output,
				const mozgVector<mozgflt,mozgint>& vector);
template ofstream& operator << (ofstream& output,
				const mozgVector<mozgint,mozgint>& vector);

// It is overloading "file_stream >> [mozgVector object]"
template <class vec_t, class vec_size_t>
ifstream& operator >> (ifstream& input,
		       const mozgVector<vec_t,vec_size_t>& vector)
{
  vec_size_t size = vector.size ();

  vec_t* temp = vector.vec ();

  while (size--)
    input >> (*temp++);

  return input;
}
template ifstream& operator >> (ifstream& input,
				const mozgVector<mozgflt,mozgint>& vector);
template ifstream& operator >> (ifstream& input,
				const mozgVector<mozgint,mozgint>& vector);

// It shows vector
template <class vec_t, class vec_size_t>
ostream& operator << (ostream& output,
		      const mozgVector<vec_t,vec_size_t>& vector)
//void  show (const mozgVector<vec_t,vec_size_t>& vector)
{
  vec_t* temp = vector.vec ();

  vec_size_t size = vector.size ();

  while (size--) output << (*temp++) << ' ';

  return output;
}
template ostream& operator << (ostream& output,
			       const mozgVector<mozgflt,mozgint>& vector);
template ostream& operator << (ostream& output,
			       const mozgVector<mozgint,mozgint>& vector);

// It returns min element
template <class vec_t, class vec_size_t> 
vec_t  min (const mozgVector<vec_t,vec_size_t>& vector)
{
  vec_t* a = vector.vec (),
         setmin;

  vec_size_t size = vector.size ();

  for (setmin = a[--size]; size;
       setmin = setmin > a[--size] ? a[size] : setmin);

  return setmin;
}
template mozgflt min (const mozgVector<mozgflt,mozgint>&);
template mozgint min (const mozgVector<mozgint,mozgint>&);

// It returns max element
template <class vec_t, class vec_size_t> 
vec_t  max (const mozgVector<vec_t,vec_size_t>& vector)
{
  vec_t* a   = vector.vec (),
         setmax;

  vec_size_t size = vector.size ();

  for (setmax = a[--size]; size;
       setmax = setmax < a[--size] ? a[size] : setmax);

  return setmax;
}
template mozgflt max (const mozgVector<mozgflt,mozgint>&);
template mozgint max (const mozgVector<mozgint,mozgint>&);

// It subtract vector2 from vector1
template <class vec_t, class vec_size_t> 
void  subtract (const vec_t* vector1,
		const mozgVector<vec_t,vec_size_t>& vector2,
		mozgVector<vec_t,vec_size_t>* result_vec)
{
  vec_size_t size = vector2.size ();

  const vec_t* temp1 = vector1,
             * temp2 = vector2.vec ();
  vec_t* temp3 = result_vec->vec ();

  while (size--) *temp3++ = *temp1++ - *temp2++;  
}
template void subtract (const mozgflt*,
			const mozgVector<mozgflt,mozgint>&,
			mozgVector<mozgflt,mozgint>*);
template void subtract (const mozgint*,
			const mozgVector<mozgint,mozgint>&,
			mozgVector<mozgint,mozgint>*);

// It subtract vector2 from vector1
template <class vec_t, class vec_size_t> 
void subtract (const mozgVector<vec_t,vec_size_t>& vector1,
	       const vec_t* vector2,
	       mozgVector<vec_t,vec_size_t>* result_vec)
{
  vec_size_t size = vector1.size ();

  const vec_t* temp1 = vector1.vec (),
             * temp2 = vector2;
  vec_t* temp3 = result_vec->vec ();

  while (size--) *temp3++ = *temp1++ - *temp2++;
}
template void subtract (const mozgVector<mozgflt,mozgint>&,
			const mozgflt*,			
			mozgVector<mozgflt,mozgint>*);
template void subtract (const mozgVector<mozgint,mozgint>&,
			const mozgint*,			
			mozgVector<mozgint,mozgint>*);

// It subtract vector2 from vector1
template <class vec_t, class vec_size_t> 
void subtract (const mozgVector<vec_t,vec_size_t>& vector1,
	       const mozgVector<vec_t,vec_size_t>& vector2,
	       mozgVector<vec_t,vec_size_t>* result_vec)
{
  vec_size_t size = vector1.size ();

#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "subtract(const mozgVector<vec_t,vec_size_t>& vector1,\n"
	 "         const mozgVector<vec_t,vec_size_t>& vector2,\n"
	 "         mozgVector<vec_t,vec_size_t>* result_vec):\n");
  if (size != vector2.size ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     vector1 and vector2 SIZES MISMATCH !!!"));
  if (size != result_vec->size ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     vector1 and result_vec"
			     " SIZES MISMATCH !!!"));
#endif

  vec_t* temp1 = vector1.vec (),
       * temp2 = vector2.vec (),
       * temp3 = result_vec->vec ();

  while (size--) *temp3++ = *temp1++ - *temp2++;
}
template void subtract (const mozgVector<mozgflt,mozgint>&,
			const mozgVector<mozgflt,mozgint>&,
			mozgVector<mozgflt,mozgint>*);
template void subtract (const mozgVector<mozgint,mozgint>&,
			const mozgVector<mozgint,mozgint>&,
			mozgVector<mozgint,mozgint>*);

// It returns inner(scalar) product of two vectors
template <class vec_t, class vec_size_t> 
void multiply (const mozgVector<vec_t,vec_size_t>& vector1,
	       const mozgVector<vec_t,vec_size_t>& vector2,
	       vec_t * product)
{
  vec_size_t size = vector1.size ();

#ifdef Mozg_BEFORE_EXCEPTION
  if (size != vector2.size ())
    before_exception ("multiply(const mozgVector<vec_t,vec_size_t>& vector1,\n"
		      "         const mozgVector<vec_t,vec_size_t>& vector2,\n"
		      "         vec_t * product):\n"
                      "     vector1 and vector2 SIZES MISMATCH !!!");
#endif

  vec_t  sum = 0,
       * v1  = vector1.vec (),
       * v2  = vector2.vec ();

  while (size--) sum += *v1++ * *v2++;

  *product = sum;
}
template void multiply (const mozgVector<mozgflt,mozgint>&,
			const mozgVector<mozgflt,mozgint>&,
			mozgflt* product);
template void multiply (const mozgVector<mozgint,mozgint>&,
			const mozgVector<mozgint,mozgint>&,
			mozgint* product);

// It puts number to all copy's elements
template <class vec_t, class vec_size_t> 
void copy (const vec_t number,
	   mozgVector<vec_t,vec_size_t>* copy)
{
  vec_size_t size = copy->size ();

  vec_t* cp = copy->vec ();

  while (size--) *cp++ = number;
}
template void copy (const mozgflt,
		    mozgVector<mozgflt,mozgint>* copy);
template void copy (const mozgint,
		    mozgVector<mozgint,mozgint>* copy);

// It copies orig_vector to copy
template <class vec_t, class vec_size_t> 
void copy (const vec_t* orig_vector,
	   mozgVector<vec_t,vec_size_t>* copy)
{
  const vec_t* or = orig_vector;

  vec_size_t size = copy->size ();

  vec_t* cp = copy->vec ();

  while (size--) *cp++ = *or++;
}
template void copy (const mozgflt*,
		    mozgVector<mozgflt,mozgint>* copy);
template void copy (const mozgint*,
		    mozgVector<mozgint,mozgint>* copy);

// It copies vector
template <class vec_t, class vec_size_t> 
void copy (const mozgVector<vec_t,vec_size_t>& orig,
	   mozgVector<vec_t,vec_size_t>* copy)
{
  vec_t* or = orig.vec ();

  vec_size_t size = orig.size ();

  vec_t* cp = copy->vec ();

  while (size--) *cp++ = *or++;
}
template void copy (const mozgVector<mozgflt,mozgint>& orig,
		    mozgVector<mozgflt,mozgint>* copy);
template void copy (const mozgVector<mozgint,mozgint>& orig,
		    mozgVector<mozgint,mozgint>* copy);

// It zeroes vector
template <class vec_t, class vec_size_t> 
void zero (mozgVector<vec_t,vec_size_t>* vector)
{
  vec_t* a = vector->vec ();

  vec_size_t size = vector->size ();

  while (size--) *a++ = 0;
}
template void zero (mozgVector<mozgflt,mozgint>* vector);
template void zero (mozgVector<mozgint,mozgint>* vector);

// It sortes vector by increase
template <class vec_t, class vec_size_t> 
void sort (mozgVector<vec_t,vec_size_t>* vector)
{
  vec_t* a = vector->vec (),
     temp;;

  vec_size_t size = vector->size (),
     j;

  short flag = 1;

  while ( size-- && flag)
    {
      flag = 0;
      j = size;
      while (j--)
	{
	  if ( *a > *(a + 1) )
	    {
	      temp     = *a;
	      *a       = *(a + 1);
	      *(a + 1) = temp;
	      flag = 1;
	    }
	  a++;
	}
    }
}
template void sort (mozgVector<mozgflt,mozgint>*);
template void sort (mozgVector<mozgint,mozgint>*);

// It makes indirect adresing ranked by increasing massive
template <class vec_t, class vec_size_t> 
void rank (const mozgVector<vec_t,vec_size_t>& vector,
	   mozgVector<vec_size_t,vec_size_t>* rank_vec)
{
  vec_size_t  size = vector.size (),
              i    = vector.size (),
              k,
              j;

  vec_t*      vec         = vector.vec ();
  vec_size_t* rank_vector = rank_vec->vec ();

  rank_vector += size - 1;
  while (--i) *rank_vector-- = i;

  while (--size)
    {
      j = size;
      while (--j)
	if (vec[rank_vector[size]]
	    < vec[rank_vector[j]])
	  {
	    k                 = rank_vector[size];
	    rank_vector[size] = rank_vector[j];
	    rank_vector[j]    = k;
	  }
    }
}
template void rank (const mozgVector<mozgflt,mozgint>&,
		    mozgVector<mozgint,mozgint>*);
template void rank (const mozgVector<mozgint,mozgint>&,
		    mozgVector<mozgint,mozgint>*);

// It randomizes vector
template <class vec_t, class vec_size_t> 
void randomize (mozgVector<vec_t,vec_size_t>* vector,
		const vec_t sigma,
		const vec_t mean)
{
  vec_t* temp = vector->vec ();
  vec_size_t size = vector->size ();

  while (size--) *temp++ = mean + gauss (sigma);
}
template  void randomize (mozgVector<mozgflt,mozgint>*,
			  const mozgflt,
			  const mozgflt);

/*****************************************************************************/

// It is overloading "file_stream << [mozgMatrix object]"
template <class matr_t, class matr_size_t>
ofstream& operator << (ofstream& output,
		       const mozgMatrix<matr_t,matr_size_t>& matr)
{
  matr_t* b = matr[0];

  matr_size_t j,
              num_rows    = matr.rows (),
              num_columns = matr.cols ();

  while (num_rows--)
    {
      j = num_columns;
      while (j--) output << ' ' << (*b++);
      output << endl;
    }
  return output;
}
template ofstream& operator << (ofstream& output,
				const mozgMatrix<mozgflt,mozgint>& matr);
template ofstream& operator << (ofstream& output,
				const mozgMatrix<mozgint,mozgint>& matr);

// It is overloading "file_stream >> [mozgMatrix object]"
template <class matr_t, class matr_size_t>
ifstream& operator >> (ifstream& input,
		       const mozgMatrix<matr_t,matr_size_t>& matr)
{
  matr_t* b = matr[0];

  matr_size_t j,
              num_rows    = matr.rows (),
              num_columns = matr.cols ();

  while (num_rows--)
    {
      j = num_columns;
      while (j--) input >> (*b++);
    }
  return input;
}
template ifstream& operator >> (ifstream& output,
				const mozgMatrix<mozgflt,mozgint>& matr);
template ifstream& operator >> (ifstream& output,
				const mozgMatrix<mozgint,mozgint>& matr);

// It shows matrix
template <class matr_t, class matr_size_t>
ostream& operator << (ostream& output,
		      const mozgMatrix<matr_t,matr_size_t>& matr)
{
  matr_t* b = matr[0];

  matr_size_t j,
              num_rows    = matr.rows (),
              num_columns = matr.cols ();

  while (num_rows--)
    {
      j = num_columns;
      while (j--) output << ' ' << (*b++);
      output << endl;
    }
  return output;
}
template ostream& operator << (ostream& output,
			       const mozgMatrix<mozgflt,mozgint>& matr);
template ostream& operator << (ostream& output,
			       const mozgMatrix<mozgint,mozgint>& matr);

// It zeroes matr
template <class matr_t, class matr_size_t> 
void zero (mozgMatrix<matr_t,matr_size_t>* matr)
{
  matr_t* b = (*matr)[0];

// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = matr->rows () * matr->cols ();

  while (size--)  *b++ = (matr_t)0.0;
}
template void zero<mozgflt,mozgint> (mozgMatrix<mozgflt,mozgint>*);
template void zero<mozgint,mozgint> (mozgMatrix<mozgint,mozgint>*);

// It makes matrix copy
template <class matr_t, class matr_size_t> 
void copy (const mozgMatrix<matr_t,matr_size_t>& orig,
	   mozgMatrix<matr_t,matr_size_t>* copy)
{
#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "copy (const mozgMatrix<matr_t,matr_size_t>& orig,\n"
	 "      mozgMatrix<matr_t,matr_size_t>* copy):\n");
  if (orig.rows () != copy->rows ()) 
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     orig.rows and copy->rows MISMATCH !!!"));
  if (orig.cols () != copy->cols ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     orig.cols and copy->cols MISMATCH !!!"));
#endif


// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = orig.rows () * orig.cols ();

  matr_t*  or = orig[0],
        *  cp = (*copy)[0];

  while (size--) *cp++ = *or++;
}
template void copy (const mozgMatrix<mozgflt,mozgint>&,
		    mozgMatrix<mozgflt,mozgint>*);
template void copy (const mozgMatrix<mozgint,mozgint>&,
		    mozgMatrix<mozgint,mozgint>*);

template <class matr_t, class matr_size_t> 
void copy (matr_t*  orig,
	   mozgMatrix<matr_t,matr_size_t>* copy)
{
// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = copy->rows () * copy->cols ();

  matr_t* cp = (*copy)[0];

  while (size--) *cp++ = *orig++;
}
template void copy (mozgflt*  orig,
		    mozgMatrix<mozgflt,mozgint>*);
//template void copy (mozgflt*  orig,
//		    mozgMatrix<mozgflt,mozgint>*);
//                  
// It addes matrices
template <class matr_t, class matr_size_t> 
void sum (const mozgMatrix<matr_t,matr_size_t>& matrix1,
	  const mozgMatrix<matr_t,matr_size_t>& matrix2,
	  mozgMatrix<matr_t,matr_size_t>* sum_matr)
{
#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "sum(const mozgMatrix<matr_t,matr_size_t>& matrix1,\n"
	 "    const mozgMatrix<matr_t,matr_size_t>& matrix2,\n"
	 "    mozgMatrix<matr_t,matr_size_t>* sum_matr):\n");
  if (matrix1.rows () != matrix2.rows () ) 
    before_exception (strcat(mozg_VecMatr_func_name,
			     "matrix1.rows and matrix2.rows MISMATCH !!!"));
  if (matrix1.cols () != matrix2.cols () )
    before_exception (strcat(mozg_VecMatr_func_name,
			     "matrix1.cols and matrix2.cols MISMATCH !!!"));
  if (matrix1.rows () != sum_matr->rows () )
    before_exception (strcat(mozg_VecMatr_func_name,
			     "matrix1.row and sum_matr->rows MISMATCH !!!"));
  if (matrix1.cols () != sum_matr->cols () )
    before_exception (strcat(mozg_VecMatr_func_name,
			     "matrix1.cols and sum_matr->cols MISMATCH !!!"));
#endif

  matr_t* temp1 = matrix1[0],
        * temp2 = matrix2[0], 
        * temp3 = (*sum_matr)[0];

// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = matrix1.rows () * matrix1.cols ();

  while (size--) *temp3++ = *temp1++ + *temp2++;
}
template void sum (const mozgMatrix<mozgflt,mozgint>&,
		   const mozgMatrix<mozgflt,mozgint>&,
		   mozgMatrix<mozgflt,mozgint>*);
template void sum (const mozgMatrix<mozgint,mozgint>&,
		   const mozgMatrix<mozgint,mozgint>&,
		   mozgMatrix<mozgint,mozgint>*);

// It addes extra_matr to matr
template <class matr_t, class matr_size_t>
mozgMatrix<matr_t,matr_size_t>&
mozgMatrix<matr_t,matr_size_t>::
operator += (const mozgMatrix<matr_t,matr_size_t>& extra_matr)
{
#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "operator += (const mozgMatrix<matr_t,matr_size_t>&extra_matr):\n");
  if (rows () != extra_matr.rows ())
    before_exception(strcat(mozg_VecMatr_func_name,
			    "     this->rows and extra_matr.rows"
			    " MISMATCH !!!"));
  if (cols () != extra_matr.cols ())
    before_exception(strcat(mozg_VecMatr_func_name,
			    "     this->cols and extra_matr.cols"
			    " MISMATCH !!!"));
#endif

  matr_t* temp1 = (*this)[0],
        * temp2 = extra_matr[0];

// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = this->rows () * this->cols ();

  while (size--) *temp1++ += *temp2++;

  return *this;
}
template mozgMatrix<mozgflt,mozgint>&
mozgMatrix<mozgflt,mozgint>::operator += (const mozgMatrix<mozgflt,mozgint>&);
template mozgMatrix<mozgint,mozgint>&
mozgMatrix<mozgint,mozgint>::operator += (const mozgMatrix<mozgint,mozgint>&);

// It multiplies matrix on number
template <class matr_t, class matr_size_t> 
void multiply (const matr_size_t d,
	       mozgMatrix<matr_t,matr_size_t>& matrix)
{
  matr_t* temp = matrix[0];

// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = matrix.rows () * matrix.cols ();

  while (size--)
      *temp++ *= d;
}
template void multiply (const mozgint,
			mozgMatrix<mozgflt,mozgint>& matrix);
template void multiply (const mozgint,
			mozgMatrix<mozgint,mozgint>& matrix);

// Matrice product
template <class matr_t, class matr_size_t> 
void multiply (const mozgMatrix<matr_t,matr_size_t>& matrix1,
	       const mozgMatrix<matr_t,matr_size_t>& matrix2,
	       mozgMatrix<matr_t,matr_size_t>* prod_matr)
{
#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "multiply (const mozgMatrix<matr_t,matr_size_t>&matrix1,\n"
	 "          const mozgMatrix<matr_t,matr_size_t>&matrix2,\n"
	 "          mozgMatrix<matr_t,matr_size_t>*prod_matr):\n");
  if (matrix1.rows () != prod_matr->rows ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     matrix1.rows and prod_matr->rows"
			     " MISMATCH !!!"));
  if  (matrix1.cols () != matrix2.rows ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     matrix1.cols and matrix2.rows"
			     " MISMATCH !!!"));
  if (matrix2.cols () != prod_matr->cols ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     matrix2.cols and prod_matr->cols"
			     " MISMATCH !!!"));
#endif

  matr_size_t i,
              j,
              k,
              rows1 = matrix1.rows (),
              cols1 = matrix1.cols (),
              cols2 = matrix2.cols ();

  matr_t** matr1 = matrix1.matr (),
        ** matr2 = matrix2.matr (),
        *  temp1,
        *  temp2,
        *  temp3 = (*prod_matr)[0];

  for (i=rows1; i--;)
    {
      for (k=0; k < cols2; k++)
	{
	  *temp3 = 0;
	  temp2 = &(matr2[0][k]);
	  temp1 = *matr1++;
	  for (j=cols1; j--;)
	    {
	      *temp3 += *temp1++ * *temp2;
	       temp2 += cols2;
	    }
	  temp3++;
	}
    }
}
template void multiply (const mozgMatrix<mozgflt,mozgint>&,
			const mozgMatrix<mozgflt,mozgint>&,
			mozgMatrix<mozgflt,mozgint>*);
template void multiply (const mozgMatrix<mozgint,mozgint>&,
			const mozgMatrix<mozgint,mozgint>&,
			mozgMatrix<mozgint,mozgint>*);

// Transpon matrix
template <class matr_t, class matr_size_t> 
void transpose (const mozgMatrix<matr_t,matr_size_t>& origin,
		mozgMatrix<matr_t,matr_size_t>* transpon)
{
#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "transpose (const mozgMatrix<matr_t,matr_size_t>& origin,\n"
	 "           mozgMatrix<matr_t,matr_size_t>* transpon):\n");
  if (origin.rows () != transpon->rows ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     origin.rows and transpon->rows"
			     " MISMATCH !!!"));
  if (origin.cols () != transpon->cols ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     origin.cols and transpon->cols"
			     " MISMATCH !!!"));
#endif

  matr_t*  temp1 = origin[0],
        *  temp2 = (*transpon)[0];

  matr_size_t i,
      j,
      num_rows = origin.rows (),
      num_cols = origin.cols ();


  for (i=num_rows; i--;)
    {
      temp2 = (*transpon)[i];
      for (j=num_cols; j--;)
	{
	  *temp2 = *temp1++;
	   temp2 += num_cols;
	}
    }
}
template void transpose (const mozgMatrix<mozgflt,mozgint>&,
			 mozgMatrix<mozgflt,mozgint>*);
template void transpose (const mozgMatrix<mozgint,mozgint>&,
			 mozgMatrix<mozgint,mozgint>*);

// It randomizes matrix
template <class matr_t, class matr_size_t>
void randomize (mozgMatrix<matr_t,matr_size_t>* matrix,
		const matr_t sigma,
		const matr_t mean)
{
  matr_t* temp = (*matrix)[0];

// WARNING !!! make sure the size don't go over the type size !!!
  matr_size_t size = matrix->rows () * matrix->cols ();

  while (size--) *temp++ = gauss (sigma) + mean;
}
template void randomize<mozgflt,mozgint> (mozgMatrix<mozgflt,mozgint>*,
					  const mozgflt,
					  const mozgflt);


/*****************************************************************************/

// Computes out product v1 and v2
template <class vec_t, class vec_size_t, class matr_t, class matr_size_t>
void multiply (const mozgVector<vec_t,vec_size_t>&  v1,
	       const mozgVector<vec_t,vec_size_t>&  v2,
	       mozgMatrix<matr_t,matr_size_t>* matrix)
{
  matr_t* temp3 = matrix->matr () [0];

  vec_t* temp1 = v1.vec (),
       * temp2;

  vec_size_t size1 = v1.size (),
             size2 = v2.size (),
             i;

#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "multiply (const mozgVector<vec_t,vec_size_t>& v1,\n"
	 "          const mozgVector<vec_t,vec_size_t>& v2,\n"
	 "          mozgMatrix<matr_t,matr_size_t>* matrix):\n");
  if (size1 != matrix->rows ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     v1.size and matrix->rows MISMATCH !!!"));
  if (size2 != matrix->cols ())
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     v2.size and matrix->cols MISMATCH !!!"));
#endif

  while (size1--)
    {
      temp2 = v2.vec ();
      i = size2;
      while (i--)
	*temp3++ = *temp1 * *temp2++;
      temp1++;
    }
}
template void multiply(const mozgVector<mozgflt,mozgint>&,
		       const mozgVector<mozgflt,mozgint>&,
		       mozgMatrix<mozgflt,mozgint>*);
template void multiply(const mozgVector<mozgint,mozgint>&,
		       const mozgVector<mozgint,mozgint>&,
		       mozgMatrix<mozgint,mozgint>*);

// It returns product matrix row and vector
template <class vec_t, class vec_size_t, class matr_t, class matr_size_t>
void multiply (const mozgMatrix<matr_t,matr_size_t>& matrix,
	       const mozgVector<vec_t,vec_size_t>& vector,
	       mozgVector<vec_t,vec_size_t>*  product_vector)
{
  matr_t* temp1 = matrix[0];
  vec_t* vec = vector.vec (),
       * result_vec = product_vector->vec (),
       * temp2,
         sum;

  matr_size_t i,
      num_rows = matrix.rows (),
      num_cols = matrix.cols ();

#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "multiply (const mozgMatrix<matr_t,matr_size_t>& matrix,\n"
	 "          const mozgVector<vec_t,vec_size_t>& vector,\n"
	 "          mozgVector<vec_t,vec_size_t>* product_vector):\n");
  if (vector.size () != num_cols)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     vector.size and matrix.cols MISMATCH !!!"));
  if (product_vector->size () != num_rows)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "     product_vector->size and matrix.rows"
			     " MISMATCH !!!"));
#endif

  while (num_rows--)
    {
      i = num_cols;
      sum = (matr_t)0.;
      temp2 = vec;
      while (i--) sum += *temp1++ * *temp2++;
      *result_vec++ = sum;
    }
}
template void multiply (const mozgMatrix<mozgflt,mozgint>&,
			const mozgVector<mozgflt,mozgint>&,
			mozgVector<mozgflt,mozgint>*);
template void multiply (const mozgMatrix<mozgint,mozgint>&,
			const mozgVector<mozgint,mozgint>&,
			mozgVector<mozgint,mozgint>*);

// It returns product matrix row and vector for the RBF out function
template <class vec_t, class vec_size_t, class matr_t, class matr_size_t>
void multiply (const mozgMatrix<matr_t,matr_size_t>& matrix,
	       const mozgVector<vec_t,vec_size_t>& vector,
	       const mozgMatrix<matr_t,matr_size_t>* RBFmatrix,
	       mozgVector<vec_t,vec_size_t>* result_vector)
{
  matr_t* temp1 = matrix[0],
       * RBFtemp = (*RBFmatrix)[0];

  vec_t* vec = vector.vec (),
       * result_vec = result_vector->vec (),
       * temp2,
         sum;

  matr_size_t i,
      num_rows = matrix.rows (),
      num_cols = matrix.cols ();

#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "multiply (const mozgMatrix<matr_t,matr_size_t>& matrix,\n"
	 "          const mozgVector<vec_t,vec_size_t>& vector,\n"
	 "          const mozgMatrix<matr_t,matr_size_t>* RBFmatrix,\n"
	 "          mozgVector<vec_t,vec_size_t>* result_vector):\n");  
  if (vector.size () != num_cols)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "vector.size and matrix.cols MISMATCH !!!"));
  if (result_vector->size () != num_rows)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "result_vector->size and matrix.rows"
			     " MISMATCH !!!"));
  if (RBFmatrix->rows () != num_rows)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "RBFmatrix->rows and matrix.rows MISMATCH !!!"));
  if (RBFmatrix->cols () != num_cols)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "RBFmatrix->cols and matrix.cols MISMATCH !!!"));
#endif

  while (num_rows--)
    {
      i = num_cols;
      sum = 0;
      temp2 = vec;
      while (i--)
	{
	  *RBFtemp = *temp1++ - *temp2++;
	  sum += *RBFtemp * *RBFtemp;
	  RBFtemp++;
	}
      *result_vec++ = sum;
    }
}
template void multiply (const mozgMatrix<mozgflt,mozgint>&,
			const mozgVector<mozgflt,mozgint>&,
			const mozgMatrix<mozgflt,mozgint>*,
			mozgVector<mozgflt,mozgint>*);
template void multiply (const mozgMatrix<mozgint,mozgint>&,
			const mozgVector<mozgint,mozgint>&,
			const mozgMatrix<mozgint,mozgint>*,
			mozgVector<mozgint,mozgint>*);

// It returns product matrix column and vector
template <class vec_t, class vec_size_t, class matr_t, class matr_size_t>
void multiply (const mozgVector<vec_t,vec_size_t>& vector,
	       const mozgMatrix<matr_t,matr_size_t>& matrix,
	       mozgVector<vec_t,vec_size_t>* product_vector)
{
  matr_t* first_row = matrix[0],
        * temp1;
  vec_t* vec = vector.vec (),
       * temp2,
       * result_vec = product_vector->vec (),
         sum = 0;

  matr_size_t num_rows = matrix.rows (),
      num_cols = matrix.cols (),
      i = num_cols,
      j;

#ifdef Mozg_BEFORE_EXCEPTION
  strcpy(mozg_VecMatr_func_name,
	 "multiply (const mozgVector<vec_t,vec_size_t>& vector,\n"
	 "          const mozgMatrix<matr_t,matr_size_t>& matrix,\n"
	 "          mozgVector<vec_t,vec_size_t>* product_vector):\n");
  if (vector.size () != num_rows)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "vector.size and matrix.rows MISMATCH !!!"));
  if (product_vector->size () != num_cols)
    before_exception (strcat(mozg_VecMatr_func_name,
			     "product_vector->size and matrix.cols"
			     " MISMATCH !!!"));
#endif

  while (i--)
    {
      j = num_rows;
      sum = 0;
      temp2 = vec;
      temp1 = first_row++;
      while (j--)
	{
	  sum += *temp1 * *temp2++;
	  temp1 += num_cols;
	}
      *result_vec++ = sum;
    }
}
template void multiply (const mozgVector<mozgflt,mozgint>&,
			const mozgMatrix<mozgflt,mozgint>&,
			mozgVector<mozgflt,mozgint>*);
template void multiply (const mozgVector<mozgint,mozgint>&,
			const mozgMatrix<mozgint,mozgint>&,
			mozgVector<mozgint,mozgint>*);


/*****************************************************************************/

// Random number generator of Park and Miller
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

mozgflt ran1(long* idum)
/* Call with negative integer to initialize ;thereafter do not alter \
idum between successive deviates in a sequence. */
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  mozgflt temp;

  if ( *idum <= 0 || !iy ) {
    if ( -(*idum) < 1 ) *idum=1;
    else *idum = -(*idum);
    for ( j=NTAB+7 ; j >= 0 ; j-- ) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if ( *idum < 0 ) *idum += IM;
      if ( j < NTAB ) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if ( *idum < 0 ) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ( (temp=AM*iy) > RNMX ) return RNMX;
  else return temp;
}

// It returns random regulary distributed in [argmin,argmax] number
mozgflt random_number(const mozgflt argmin,
		      const mozgflt argmax)
{
    mozgflt r;
    static long idum = -1;

    r = ran1(&idum);
    return r*fabs(argmax-argmin) + argmin;
}


// It returns a normally distributed deviate with zero mean and unit variance
mozgflt gasdev ()
{
  static short iset=0;
  static mozgflt gset;
  mozgflt fac,rsq,v1,v2;

  if ( iset==0 )
    {
      do
	{
	  v1=2.0*random_number(0.0,1.0)-1.0;
	  v2=2.0*random_number(0.0,1.0)-1.0;
	  rsq=v1*v1+v2*v2;
	}
      while ( rsq >=1.0 || rsq == 0.0 );
      fac=sqrt(-2.0*log(rsq)/rsq);
      gset=v1*fac;
      iset=1;
      return v2*fac;
    }
  else
    {
      iset=0;
      return gset;
    }
}

// It returns random normally distributed values with zero mean and 
//sigma dispersion      
mozgflt gauss(const mozgflt sigma)
{
    return  gasdev () * sigma;
}

// It returns random normally distributed values with sigma dispersion 
// and mean value mean
mozgflt gauss(const mozgflt sigma,
	      const mozgflt mean)
{
    return  gasdev () * sigma + mean;
}



// Return random regular distributed signum
mozgflt rand_signum ()
{
    mozgflt sign=1;

    if ( random_number(0.,2.) > 1 ) sign=1;
    else sign=-1;
    return sign;
}
